@@ -16,7 +16,7 @@ from .constants import *
 
 
 # recommended range from the GM2 specs
-# note: the "Applause" at pitch 88 of the orchestra drum set is ignored, increase to 89 if you need it
+# note: we ignore the "Applause" at pitch 88 of the orchestra drum set, increase to 89 if you need it
 DRUM_PITCH_RANGE = range(27, 88)
 
 
@@ -32,17 +32,19 @@ class MuMIDI(MIDITokenizer):
     :param nb_velocities: number of velocity bins
     :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo)
     :param sos_eos_tokens: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary
+    :param mask: will add a MASK token to the vocabulary (default: False)
     :param params: can be a path to the parameter (json encoded) file or a dictionary
     :param drum_pitch_range: range of used MIDI pitches for drums exclusively
     """
     def __init__(self, pitch_range: range = PITCH_RANGE, beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
                  nb_velocities: int = NB_VELOCITIES, additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-                 sos_eos_tokens: bool = False, params=None, drum_pitch_range: range = DRUM_PITCH_RANGE):
+                 sos_eos_tokens: bool = False, mask: bool = False, params=None,
+                 drum_pitch_range: range = DRUM_PITCH_RANGE):
         additional_tokens['Rest'] = False
         self.drum_pitch_range = drum_pitch_range
         # used in place of positional encoding
         self.max_bar_embedding = 60  # this attribute might increase during encoding
-        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, params)
+        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, mask, params)
 
     def save_params(self, out_dir: Union[str, Path, PurePath]):
         """ Override the parent class method to include additional parameter drum pitch range
@@ -299,15 +301,18 @@ class MuMIDI(MIDITokenizer):
         """
         raise NotImplementedError('tokens_to_track not implemented for Octuple, use tokens_to_midi instead')
 
-    def _create_vocabulary(self, sos_eos_tokens: bool = False) -> Vocabulary:
+    def _create_vocabulary(self, sos_eos_tokens: bool = None) -> Vocabulary:
         """ Creates the Vocabulary object of the tokenizer.
         See the docstring of the Vocabulary class for more details about how to use it.
-        NOTE: token index 0 is often used as a padding index during training
+        NOTE: token index 0 is often used as a padding index during training TODO several vocabs + note in readme
 
-        :param sos_eos_tokens: will include Start Of Sequence (SOS) and End Of Sequence (tokens)
+        :param sos_eos_tokens: DEPRECIATED, will include Start Of Sequence (SOS) and End Of Sequence (tokens)
         :return: the vocabulary object
         """
-        vocab = Vocabulary({'PAD_None': 0})
+        if sos_eos_tokens is not None:
+            print(f'\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
+                  f'_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
+        vocab = Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask)
 
         # PITCH
         vocab.add_event(f'Pitch_{i}' for i in self.pitch_range)
@@ -342,10 +347,6 @@ class MuMIDI(MIDITokenizer):
         # PROGRAM
         vocab.add_event(f'Program_{program}' for program in range(-1, 128))
 
-        # SOS & EOS
-        if sos_eos_tokens:
-            vocab.add_sos_eos()
-
         # BAR --- MUST BE LAST IN DIC AS THIS MIGHT BE INCREASED
         vocab.add_event('Bar_None')  # new bar token
         vocab.add_event(f'Bar_{i}' for i in range(self.max_bar_embedding))  # bar embeddings (positional encoding)
